@@ -156,6 +156,29 @@ local function prepare_dkim_signing(N, task, settings)
     is_local = true
   end
 
+  local has_pre_result = task:has_pre_result()
+  if has_pre_result then
+    local metric_action = task:get_metric_action()
+
+    if metric_action == 'reject' or metric_action == 'drop' then
+      -- No need to sign what we are already rejecting/dropping
+      lua_util.debugm(N, task, 'task result is already %s, no need to sign', metric_action)
+      return false,{}
+    end
+
+    if metric_action == 'soft reject' then
+      -- Same here, we are going to delay an email, signing is just a waste of time
+      lua_util.debugm(N, task, 'task result is %s, skip signing', metric_action)
+      return false,{}
+    end
+
+    -- For spam actions, there is no clear distinction
+    if metric_action ~= 'no action' and type(settings.skip_spam_sign) == 'boolean' and settings.skip_spam_sign then
+      lua_util.debugm(N, task, 'task result is %s, no need to sign', metric_action)
+      return false,{}
+    end
+  end
+
   if settings.sign_authenticated and auser then
     lua_util.debugm(N, task, 'user is authenticated')
     is_authed = true
@@ -596,6 +619,7 @@ exports.sign_using_vault = function(N, task, settings, selectors, sign_func, err
 
   local full_url = string.format('%s/v1/%s/%s',
       settings.vault_url, settings.vault_path or 'dkim', selectors.domain)
+  local upstream_list = lua_util.http_upstreams_by_url(rspamd_config:get_mempool(), settings.vault_url)
 
   local function vault_callback(err, code, body, _)
     if code ~= 200 then
@@ -660,6 +684,7 @@ exports.sign_using_vault = function(N, task, settings, selectors, sign_func, err
     timeout = settings.http_timeout or 5.0,
     no_ssl_verify = settings.no_ssl_verify,
     keepalive = true,
+    upstream = upstream_list and upstream_list:get_upstream_round_robin() or nil,
     headers = {
       ['X-Vault-Token'] = settings.vault_token,
     },
@@ -685,19 +710,20 @@ end
 
 exports.process_signing_settings = function(N, settings, opts)
   local lua_maps = require "lua_maps"
+  -- Used to convert plain options to the maps
+  local maps_opts = {
+    sign_networks = {'radix', 'DKIM signing networks'},
+    path_map = {'map', 'Paths to DKIM signing keys'},
+    selector_map = {'map', 'DKIM selectors'},
+    signing_table = {'glob', 'DKIM signing table'},
+    key_table = {'glob', 'DKIM keys table'},
+    vault_domains = {'glob', 'DKIM signing domains in vault'},
+    whitelisted_signers_map = {'set', 'ARC trusted signers domains'}
+  }
   for k,v in pairs(opts) do
-    if k == 'sign_networks' then
-      settings[k] = lua_maps.map_add(N, k, 'radix', 'DKIM signing networks')
-    elseif k == 'path_map' then
-      settings[k] = lua_maps.map_add(N, k, 'map', 'Paths to DKIM signing keys')
-    elseif k == 'selector_map' then
-      settings[k] = lua_maps.map_add(N, k, 'map', 'DKIM selectors')
-    elseif k == 'signing_table' then
-      settings[k] = lua_maps.map_add(N, k, 'glob', 'DKIM signing table')
-    elseif k == 'key_table' then
-      settings[k] = lua_maps.map_add(N, k, 'glob', 'DKIM keys table')
-    elseif k == 'vault_domains' then
-      settings[k] = lua_maps.map_add(N, k, 'glob', 'DKIM signing domains in vault')
+    local maybe_map = maps_opts[k]
+    if maybe_map then
+      settings[k] = lua_maps.map_add_from_ucl(v, maybe_map[1], maybe_map[2])
     elseif k == 'sign_condition' then
       local ret,f = lua_util.callback_from_string(v)
       if ret then
@@ -705,8 +731,6 @@ exports.process_signing_settings = function(N, settings, opts)
       else
         logger.errx(rspamd_config, 'cannot load sign condition %s: %s', v, f)
       end
-    elseif k == 'whitelisted_signers_map' then
-      settings[k] = lua_maps.map_add(N, k, 'set', 'ARC trusted signers domains')
     else
       settings[k] = v
     end
