@@ -1,5 +1,5 @@
 --[[
-Copyright (c) 2011-2020, Vsevolod Stakhov <vsevolod@highsecure.ru>
+Copyright (c) 2022, Vsevolod Stakhov <vsevolod@rspamd.com>
 Copyright (c) 2013-2015, Andrew Lewis <nerf@judo.za.org>
 
 Licensed under the Apache License, Version 2.0 (the "License");
@@ -136,28 +136,27 @@ local function ip_to_rbl(ip)
 end
 
 local function gen_check_rcvd_conditions(rbl, received_total)
-  local min_pos = tonumber(rbl['received_min_pos'])
-  local max_pos = tonumber(rbl['received_max_pos'])
-  local match_flags = rbl['received_flags']
-  local nmatch_flags = rbl['received_nflags']
+  local min_pos = tonumber(rbl.received_min_pos)
+  local max_pos = tonumber(rbl.received_max_pos)
+  local match_flags = rbl.received_flags
+  local nmatch_flags = rbl.received_nflags
+
   local function basic_received_check(rh)
-    if not (rh['real_ip'] and rh['real_ip']:is_valid()) then return false end
-    if ((rh['real_ip']:get_version() == 6 and rbl['ipv6']) or
-        (rh['real_ip']:get_version() == 4 and rbl['ipv4'])) and
-        ((rbl['exclude_private_ips'] and not rh['real_ip']:is_local()) or
-            not rbl['exclude_private_ips']) and ((rbl['exclude_local_ips'] and
-        not is_excluded_ip(rh['real_ip'])) or not rbl['exclude_local_ips']) then
+    if not (rh.real_ip and rh.real_ip:is_valid()) then return false end
+    if ((rh.real_ip:get_version() == 6 and rbl.ipv6) or
+        (rh.real_ip:get_version() == 4 and rbl.ipv4)) and
+        ((rbl.exclude_private_ips and not rh.real_ip:is_local()) or
+            not rbl.exclude_private_ips) and ((rbl.exclude_local_ips and
+        not is_excluded_ip(rh.real_ip)) or not rbl.exclude_local_ips) then
       return true
     else
       return false
     end
   end
-  if not (max_pos or min_pos or match_flags or nmatch_flags) then
-    return basic_received_check
-  end
-  return function(rh, pos)
-    if not basic_received_check() then return false end
-    local got_flags = rh['flags'] or E
+
+  local function positioned_received_check(rh, pos)
+    if not rh or not basic_received_check(rh) then return false end
+    local got_flags = rh.flags or E
     if min_pos then
       if min_pos < 0 then
         if min_pos == -1 then
@@ -165,7 +164,7 @@ local function gen_check_rcvd_conditions(rbl, received_total)
             return false
           end
         else
-          if pos <= (received_total - (min_pos*-1)) then
+          if pos <= (received_total - math.abs(min_pos)) then
             return false
           end
         end
@@ -175,7 +174,7 @@ local function gen_check_rcvd_conditions(rbl, received_total)
     end
     if max_pos then
       if max_pos < -1 then
-        if (received_total - (max_pos*-1)) >= pos then
+        if (received_total - math.abs(max_pos)) >= pos then
           return false
         end
       elseif max_pos > 0 then
@@ -200,6 +199,13 @@ local function gen_check_rcvd_conditions(rbl, received_total)
     end
     return true
   end
+
+
+  if not (max_pos or min_pos or match_flags or nmatch_flags) then
+    return basic_received_check
+  else
+    return positioned_received_check
+  end
 end
 
 local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt)
@@ -457,7 +463,8 @@ local function gen_rbl_callback(rule)
     local helo = task:get_helo()
 
     if not helo then
-      return false
+      -- Avoid pipeline breaking
+      return true
     end
 
     add_dns_request(task, helo, true, false, requests_table,
@@ -630,7 +637,7 @@ local function gen_rbl_callback(rule)
   local function check_rdns(task, requests_table, whitelist)
     local hostname = task:get_hostname()
     if hostname == nil or hostname == 'unknown' then
-      return false
+      return true
     end
 
     add_dns_request(task, hostname, true, false,
@@ -1034,6 +1041,7 @@ local function add_rbl(key, rbl, global_opts)
       id = rspamd_config:register_symbol{
         type = 'callback',
         callback = callback,
+        groups = {'rbl'},
         name = rbl.symbol .. '_CHECK',
         flags = table.concat(flags_tbl, ',')
       }
@@ -1060,6 +1068,7 @@ local function add_rbl(key, rbl, global_opts)
         type = 'callback',
         callback = callback,
         name = rbl.symbol,
+        groups = {'rbl'},
         group = 'rbl',
         score = 0,
         flags = table.concat(flags_tbl, ',')
@@ -1079,13 +1088,15 @@ local function add_rbl(key, rbl, global_opts)
     lua_util.debugm(N, rspamd_config, 'rule dump for %s: %s',
         rbl.symbol, rbl)
 
+    local check_sym = rbl.symbol_prefixes and rbl.symbol .. '_CHECK' or rbl.symbol
+
     if rbl.dkim then
-      rspamd_config:register_dependency(rbl.symbol, 'DKIM_CHECK')
+      rspamd_config:register_dependency(check_sym, 'DKIM_CHECK')
     end
 
     if rbl.require_symbols then
       for _,dep in ipairs(rbl.require_symbols) do
-        rspamd_config:register_dependency(rbl.symbol, dep)
+        rspamd_config:register_dependency(check_sym, dep)
       end
     end
 
@@ -1194,7 +1205,20 @@ if(opts['local_exclude_ip_map'] ~= nil) then
     'RBL exclusions map')
 end
 
-for key,rbl in pairs(opts.rbls ) do
+-- TODO: this code should be universal for all modules that use selectors to allow
+-- maps usage from selectors registered for a specific module
+if type(opts.attached_maps) == 'table' then
+  opts.attached_maps_processed = {}
+  for i,map in ipairs(opts.attached_maps) do
+    -- Store maps in the configuration table to keep lifetime track
+    opts.attached_maps_processed[i] = lua_maps.map_add_from_ucl(map)
+    if opts.attached_maps_processed[i] == nil then
+      rspamd_logger.warnx(rspamd_config, "cannot parse attached map: %s", map)
+    end
+  end
+end
+
+for key,rbl in pairs(opts.rbls) do
   if type(rbl) ~= 'table' or rbl.disabled == true or rbl.enabled == false then
     rspamd_logger.infox(rspamd_config, 'disable rbl "%s"', key)
   else
@@ -1277,14 +1301,18 @@ rspamd_config:register_symbol{
   type = 'callback',
   callback = rbl_callback_white,
   name = 'RBL_CALLBACK_WHITE',
-  flags = 'nice,empty,no_squeeze'
+  flags = 'nice,empty,no_squeeze',
+  groups = {'rbl'},
+  augmentations = {string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0)},
 }
 
 rspamd_config:register_symbol{
   type = 'callback',
   callback = rbl_callback_fin,
   name = 'RBL_CALLBACK',
-  flags = 'empty,no_squeeze'
+  flags = 'empty,no_squeeze',
+  groups = {'rbl'},
+  augmentations = {string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0)},
 }
 
 for _, w in ipairs(white_symbols) do
